@@ -4,21 +4,29 @@ use crate::constants::{
 };
 use crate::ops;
 use crate::Error;
-
 use core::{
     cmp::{Ordering::Equal, *},
     fmt,
     hash::{Hash, Hasher},
-    iter::Sum,
+    iter::{Product, Sum},
     ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign},
     str::FromStr,
 };
-#[cfg(feature = "diesel")]
+
+// Diesel configuration
+#[cfg(feature = "diesel2")]
+use diesel::deserialize::FromSqlRow;
+#[cfg(feature = "diesel2")]
+use diesel::expression::AsExpression;
+#[cfg(any(feature = "diesel1", feature = "diesel2"))]
 use diesel::sql_types::Numeric;
+
 #[allow(unused_imports)] // It's not actually dead code below, but the compiler thinks it is.
 #[cfg(not(feature = "std"))]
 use num_traits::float::FloatCore;
 use num_traits::{FromPrimitive, Num, One, Signed, ToPrimitive, Zero};
+#[cfg(feature = "rkyv")]
+use rkyv::{Archive, Deserialize, Serialize};
 
 /// The smallest value that can be represented by this decimal type.
 const MIN: Decimal = Decimal {
@@ -95,12 +103,24 @@ pub struct UnpackedDecimal {
 /// where m is an integer such that -2<sup>96</sup> < m < 2<sup>96</sup>, and e is an integer
 /// between 0 and 28 inclusive.
 #[derive(Clone, Copy)]
-#[cfg_attr(feature = "diesel", derive(FromSqlRow, AsExpression), sql_type = "Numeric")]
+#[cfg_attr(
+    all(feature = "diesel1", not(feature = "diesel2")),
+    derive(FromSqlRow, AsExpression),
+    sql_type = "Numeric"
+)]
+#[cfg_attr(feature = "diesel2", derive(FromSqlRow, AsExpression), diesel(sql_type = Numeric))]
 #[cfg_attr(feature = "c-repr", repr(C))]
 #[cfg_attr(
     feature = "borsh",
     derive(borsh::BorshDeserialize, borsh::BorshSerialize, borsh::BorshSchema)
 )]
+#[cfg_attr(
+    feature = "rkyv",
+    derive(Archive, Deserialize, Serialize),
+    archive(compare(PartialEq)),
+    archive_attr(derive(Clone, Copy, Debug))
+)]
+#[cfg_attr(feature = "rkyv-safe", archive(check_bytes))]
 pub struct Decimal {
     // Bits 0-15: unused
     // Bits 16-23: Contains "e", a value between 0-28 that indicates the scale
@@ -114,6 +134,9 @@ pub struct Decimal {
     mid: u32,
 }
 
+#[cfg(feature = "ndarray")]
+impl ndarray::ScalarOperand for Decimal {}
+
 /// `RoundingStrategy` represents the different rounding strategies that can be used by
 /// `round_dp_with_strategy`.
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
@@ -679,7 +702,7 @@ impl Decimal {
     #[inline]
     #[must_use]
     pub const fn scale(&self) -> u32 {
-        ((self.flags & SCALE_MASK) >> SCALE_SHIFT) as u32
+        (self.flags & SCALE_MASK) >> SCALE_SHIFT
     }
 
     /// Returns the mantissa of the decimal number.
@@ -719,6 +742,46 @@ impl Decimal {
         self.lo == 0 && self.mid == 0 && self.hi == 0
     }
 
+    /// Returns true if this Decimal number has zero fractional part (is equal to an integer)
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use rust_decimal::prelude::*;
+    /// # use rust_decimal_macros::dec;
+    /// #
+    /// assert_eq!(dec!(5).is_integer(), true);
+    /// // Trailing zeros are also ignored
+    /// assert_eq!(dec!(5.0000).is_integer(), true);
+    /// // If there is a fractional part then it is not an integer
+    /// assert_eq!(dec!(5.1).is_integer(), false);
+    /// ```
+    #[must_use]
+    pub fn is_integer(&self) -> bool {
+        let scale = self.scale();
+        if scale == 0 || self.is_zero() {
+            return true;
+        }
+
+        // Check if it can be divided by 10^scale without remainder
+        let mut bits = self.mantissa_array3();
+        let mut scale = scale;
+        while scale > 0 {
+            let remainder = if scale > 9 {
+                scale -= 9;
+                ops::array::div_by_u32(&mut bits, POWERS_10[9])
+            } else {
+                let power = POWERS_10[scale as usize];
+                scale = 0;
+                ops::array::div_by_u32(&mut bits, power)
+            };
+            if remainder > 0 {
+                return false;
+            }
+        }
+        true
+    }
+
     /// An optimized method for changing the sign of a decimal number.
     ///
     /// # Arguments
@@ -916,9 +979,9 @@ impl Decimal {
         if raw.scale() > MAX_PRECISION_U32 {
             let mut bits = raw.mantissa_array3();
             let remainder = match raw.scale() {
-                29 => crate::ops::array::div_by_1x(&mut bits, 1),
-                30 => crate::ops::array::div_by_1x(&mut bits, 2),
-                31 => crate::ops::array::div_by_1x(&mut bits, 3),
+                29 => ops::array::div_by_power::<1>(&mut bits),
+                30 => ops::array::div_by_power::<2>(&mut bits),
+                31 => ops::array::div_by_power::<3>(&mut bits),
                 _ => 0,
             };
             if remainder >= 5 {
@@ -997,36 +1060,54 @@ impl Decimal {
     ///
     /// ```
     /// # use rust_decimal::Decimal;
+    /// # use rust_decimal_macros::dec;
     /// #
-    /// let pi = Decimal::new(3141, 3);
-    /// let trunc = Decimal::new(3, 0);
-    /// // note that it returns a decimal
-    /// assert_eq!(pi.trunc(), trunc);
+    /// let pi = dec!(3.141);
+    /// assert_eq!(pi.trunc(), dec!(3));
+    ///
+    /// // Negative numbers are similarly truncated without rounding
+    /// let neg = dec!(-1.98765);
+    /// assert_eq!(neg.trunc(), Decimal::NEGATIVE_ONE);
     /// ```
     #[must_use]
     pub fn trunc(&self) -> Decimal {
-        let mut scale = self.scale();
-        if scale == 0 {
-            // Nothing to do
-            return *self;
-        }
         let mut working = [self.lo, self.mid, self.hi];
-        while scale > 0 {
-            // We're removing precision, so we don't care about overflow
-            if scale < 10 {
-                ops::array::div_by_u32(&mut working, POWERS_10[scale as usize]);
-                break;
-            } else {
-                ops::array::div_by_u32(&mut working, POWERS_10[9]);
-                // Only 9 as this array starts with 1
-                scale -= 9;
-            }
-        }
+        let mut working_scale = self.scale();
+        ops::array::truncate_internal(&mut working, &mut working_scale, 0);
         Decimal {
             lo: working[0],
             mid: working[1],
             hi: working[2],
-            flags: flags(self.is_sign_negative(), 0),
+            flags: flags(self.is_sign_negative(), working_scale),
+        }
+    }
+
+    /// Returns a new `Decimal` with the fractional portion delimited by `scale`.
+    /// This is a true truncation whereby no rounding is performed.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use rust_decimal::Decimal;
+    /// # use rust_decimal_macros::dec;
+    /// #
+    /// let pi = dec!(3.141592);
+    /// assert_eq!(pi.trunc_with_scale(2), dec!(3.14));
+    ///
+    /// // Negative numbers are similarly truncated without rounding
+    /// let neg = dec!(-1.98765);
+    /// assert_eq!(neg.trunc_with_scale(1), dec!(-1.9));
+    /// ```
+    #[must_use]
+    pub fn trunc_with_scale(&self, scale: u32) -> Decimal {
+        let mut working = [self.lo, self.mid, self.hi];
+        let mut working_scale = self.scale();
+        ops::array::truncate_internal(&mut working, &mut working_scale, scale);
+        Decimal {
+            lo: working[0],
+            mid: working[1],
+            hi: working[2],
+            flags: flags(self.is_sign_negative(), working_scale),
         }
     }
 
@@ -1254,6 +1335,14 @@ impl Decimal {
     /// ```
     #[must_use]
     pub fn round_dp_with_strategy(&self, dp: u32, strategy: RoundingStrategy) -> Decimal {
+        let old_scale = self.scale();
+
+        // return early if decimal has a smaller number of fractional places than dp
+        // e.g. 2.51 rounded to 3 decimal places is 2.51
+        if old_scale <= dp {
+            return *self;
+        }
+
         // Short circuit for zero
         if self.is_zero() {
             return Decimal {
@@ -1264,14 +1353,6 @@ impl Decimal {
             };
         }
 
-        let old_scale = self.scale();
-
-        // return early if decimal has a smaller number of fractional places than dp
-        // e.g. 2.51 rounded to 3 decimal places is 2.51
-        if old_scale <= dp {
-            return *self;
-        }
-
         let mut value = [self.lo, self.mid, self.hi];
         let mut value_scale = self.scale();
         let negative = self.is_sign_negative();
@@ -1720,12 +1801,12 @@ macro_rules! impl_try_from_decimal {
             stringify!($TInto),
             "`.",
         )]
-        impl core::convert::TryFrom<Decimal> for $TInto {
+        impl TryFrom<Decimal> for $TInto {
             type Error = crate::Error;
 
             #[inline]
             fn try_from(t: Decimal) -> Result<Self, Error> {
-                $conversion_fn(&t).ok_or_else(|| Error::ConversionTo(stringify!($TInto).to_string()))
+                $conversion_fn(&t).ok_or_else(|| Error::ConversionTo(stringify!($TInto).into()))
             }
         }
     };
@@ -1750,25 +1831,26 @@ impl_try_from_decimal!(u128, Decimal::to_u128, integer_docs!(true));
 // See https://github.com/rust-lang/rustfmt/issues/5062 for more information.
 #[rustfmt::skip]
 macro_rules! impl_try_from_primitive {
-    ($TFrom:ty, $conversion_fn:path) => {
+    ($TFrom:ty, $conversion_fn:path $(, $err:expr)?) => {
         #[doc = concat!(
             "Try to convert a `",
             stringify!($TFrom),
             "` into a `Decimal`.\n\nCan fail if the value is out of range for `Decimal`."
         )]
-        impl core::convert::TryFrom<$TFrom> for Decimal {
+        impl TryFrom<$TFrom> for Decimal {
             type Error = crate::Error;
 
             #[inline]
             fn try_from(t: $TFrom) -> Result<Self, Error> {
-                $conversion_fn(t).ok_or_else(|| Error::ConversionTo("Decimal".to_string()))
+                $conversion_fn(t) $( .ok_or_else(|| $err) )?
             }
         }
     };
 }
 
-impl_try_from_primitive!(f32, Self::from_f32);
-impl_try_from_primitive!(f64, Self::from_f64);
+impl_try_from_primitive!(f32, Self::from_f32, Error::ConversionTo("Decimal".into()));
+impl_try_from_primitive!(f64, Self::from_f64, Error::ConversionTo("Decimal".into()));
+impl_try_from_primitive!(&str, core::str::FromStr::from_str);
 
 macro_rules! impl_from {
     ($T:ty, $from_ty:path) => {
@@ -2272,7 +2354,7 @@ impl ToPrimitive for Decimal {
             let integer = self.to_i128();
             integer.map(|i| i as f64)
         } else {
-            let sign: f64 = if self.is_sign_negative() { -1.0 } else { 1.0 };
+            let neg = self.is_sign_negative();
             let mut mantissa: u128 = self.lo.into();
             mantissa |= (self.mid as u128) << 32;
             mantissa |= (self.hi as u128) << 64;
@@ -2282,9 +2364,24 @@ impl ToPrimitive for Decimal {
             let integral_part = mantissa / precision;
             let frac_part = mantissa % precision;
             let frac_f64 = (frac_part as f64) / (precision as f64);
-            let value = sign * ((integral_part as f64) + frac_f64);
+            let integral = integral_part as f64;
+            // If there is a fractional component then we will need to add that and remove any
+            // inaccuracies that creep in during addition. Otherwise, if the fractional component
+            // is zero we can exit early.
+            if frac_f64.is_zero() {
+                if neg {
+                    return Some(-integral);
+                }
+                return Some(integral);
+            }
+            let value = integral + frac_f64;
             let round_to = 10f64.powi(self.scale() as i32);
-            Some((value * round_to).round() / round_to)
+            let rounded = (value * round_to).round() / round_to;
+            if neg {
+                Some(-rounded)
+            } else {
+                Some(rounded)
+            }
         }
     }
 }
@@ -2514,6 +2611,28 @@ impl Ord for Decimal {
     }
 }
 
+impl Product for Decimal {
+    /// Panics if out-of-bounds
+    fn product<I: Iterator<Item = Decimal>>(iter: I) -> Self {
+        let mut product = ONE;
+        for i in iter {
+            product *= i;
+        }
+        product
+    }
+}
+
+impl<'a> Product<&'a Decimal> for Decimal {
+    /// Panics if out-of-bounds
+    fn product<I: Iterator<Item = &'a Decimal>>(iter: I) -> Self {
+        let mut product = ONE;
+        for i in iter {
+            product *= i;
+        }
+        product
+    }
+}
+
 impl Sum for Decimal {
     fn sum<I: Iterator<Item = Decimal>>(iter: I) -> Self {
         let mut sum = ZERO;
